unit uFileSearch;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, contnrs, md5;

type
  PFileData = ^TFileData;
  TFileData = record
    FName: String;
    FSize: Int64;
    FAttr: LongInt;
  end;

  { TFileSearch }

  TOnSearching = procedure(Sender: TObject; AFileCount: Integer) of object;
  TOnSearchComplete = procedure(Sender: TObject; AHashList: TFPHashList) of object;
  TFileSearch = class(TThread)
  private
    FPath: String;
    FHashList: TFPHashList;
    FFileCount: Integer;
    FNeedToBreak: Boolean;
    FOnSearching: TOnSearching;
    FOnSearchComplete: TOnSearchComplete;
    procedure FindFiles(const APath: String);
    procedure UpdateSearching;
    procedure UpdateSearchComplete;
  protected
    procedure Execute; override;
  public
    constructor Create(APath: String; AHashList: TFPHashList);
    destructor Destroy; override;
  public
    property NeedToBreak: Boolean read FNeedToBreak write FNeedToBreak;
    property OnSearching: TOnSearching read FOnSearching write FOnSearching;
    property OnSearchComplete: TOnSearchComplete read FOnSearchComplete write FOnSearchComplete;
  end;

implementation

{ TFileSearch }
procedure TFileSearch.FindFiles(const APath: String);
  procedure Search(const APath: String);
  var
    SearchRec: TSearchRec;
    Path: String;
    FileData: PFileData;
    FileHash: ShortString;
  begin
    Path := IncludeTrailingBackslash(APath);
    if FindFirst(Path + '*', faAnyFile - faDirectory, SearchRec) = 0 then
    begin
      repeat
        if FNeedToBreak then
          Break;
        New(FileData);
        FileData^.FName := Path + SearchRec.Name;
        FileData^.FSize := SearchRec.Size;
        FileData^.FAttr := SearchRec.Attr;
        FileHash := MD5Print(MD5String(LowerCase(SearchRec.Name)));
        FHashList.Add(FileHash, FileData);
        Inc(FFileCount);
        if FFileCount = 50 then
        begin
          FFileCount := 0;
          Synchronize(@UpdateSearching);
        end;
      until FindNext(SearchRec) <> 0;
      FindClose(SearchRec);
    end;

    if FindFirst(Path + '*', faAnyFile, SearchRec) = 0 then
    begin
      repeat
        if FNeedToBreak then
          Break;
        if ((SearchRec.Attr and faDirectory) <> 0) and (SearchRec.Name <> '.') and (SearchRec.Name <> '..') then
          Search(Path + SearchRec.Name)
       until FindNext(SearchRec) <> 0;
       FindClose(SearchRec);
    end;
  end;
begin
  Search(APath);
  FNeedToBreak := True;
end;

procedure TFileSearch.UpdateSearching;
begin
  if Assigned(FOnSearching) then
    FOnSearching(Self, FHashList.Count);
end;

procedure TFileSearch.UpdateSearchComplete;
begin
  if Assigned(FOnSearchComplete) then
    FOnSearchComplete(Self, FHashList);
end;

procedure TFileSearch.Execute;
begin
  FindFiles(FPath);
  Synchronize(@UpdateSearchComplete);
end;

constructor TFileSearch.Create(APath: String; AHashList: TFPHashList);
begin
  inherited Create(True);
  FPath := IncludeTrailingBackslash(APath);
  FNeedToBreak := False;
  FFileCount := 0;
  FHashList := AHashList;
end;

destructor TFileSearch.Destroy;
begin
  //
  inherited Destroy;
end;

end.

