unit umain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls, dateutils, IniFiles, uPanel, uFileSearch;

type

  { TfMain }

  TfMain = class(TForm)
    bScan: TButton;
    ListBox1: TListBox;
    pnStatus: TPanel;
    pnButtons: TPanel;
    pnPanels: TPanel;
    spMain: TSplitter;
    tmWait: TTimer;
    procedure bScanClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure tmWaitTimer(Sender: TObject);
  private
    FConfigDir: String;
    FConfigFile: String;
    FScanTime: TDateTime;
    FPanelLeft: TfrPanel;
    FPanelRight: TfrPanel;
    procedure ReadSettings;
    procedure WriteSettings;
    procedure ListBoxOnScroll(Sender: TObject);
    procedure ListBoxOnSelectionChange(Sender: TObject; {%H-}User: boolean);
    { private declarations }
  public
    { public declarations }
  end;

var
  fMain: TfMain;

implementation

{$R *.lfm}

{ TfMain }

procedure TfMain.bScanClick(Sender: TObject);
begin
  if bScan.Caption = 'Scan' then
  begin
    if (FPanelLeft.CanScan) and (FPanelRight.CanScan) then
    begin
      if Trim(UpperCase(FPanelLeft.edPath.Text)) = Trim(UpperCase(FPanelRight.edPath.Text)) then
      begin
        MessageDlg('The paths are identical! Please select a different path.', mtInformation, [mbOk], 0);
        FPanelRight.edPath.SetFocus;
        Exit;
      end;
      pnStatus.Visible := True;
      pnStatus.Caption := 'Searching for files. Please wait...';
      FScanTime := now;
      bScan.Caption := 'Cancel';
      FPanelLeft.Scan;
      FPanelRight.Scan;
      tmWait.Enabled := True;
    end;
  end
  else
  begin
    pnStatus.Caption := 'Compare folders';
    FPanelLeft.Scan(True);
    FPanelRight.Scan(True);
    bScan.Caption := 'Scan';
    pnStatus.Visible := False;
    tmWait.Enabled := False;
  end;
end;

procedure TfMain.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if bScan.Caption = 'Cancel' then
  begin
    FPanelLeft.Scan(True);
    FPanelRight.Scan(True);
  end;
  WriteSettings;
  CanClose := True;
end;

procedure TfMain.FormCreate(Sender: TObject);
begin
  FPanelLeft := TfrPanel.Create(Self);
  FPanelLeft.InitFrame(frPanelLeft, pnPanels);
  FPanelLeft.LBFiles.OnScroll := @ListBoxOnScroll;
  FPanelLeft.LBFiles.OnSelectionChange := @ListBoxOnSelectionChange;
  FPanelRight := TfrPanel.Create(Self);
  FPanelRight.InitFrame(frPanelRight, pnPanels);
  FPanelRight.LBFiles.OnScroll := @ListBoxOnScroll;
  FPanelRight.LBFiles.OnSelectionChange := @ListBoxOnSelectionChange;
  FConfigDir := GetAppConfigDir(False);
  FConfigFile := FConfigDir + 'Find.ini';
  ReadSettings;
  fMain.Width := 1000;
end;

procedure TfMain.FormDestroy(Sender: TObject);
begin
  FPanelLeft.FinalizeFrame;
  FPanelRight.FinalizeFrame;
end;

procedure TfMain.FormResize(Sender: TObject);
begin
  FPanelLeft.Width := fMain.Width div 2;
  bScan.Left := (fMain.Width - bScan.Width) div 2;
end;

procedure TfMain.tmWaitTimer(Sender: TObject);
var
  I: Integer;
  CommonCount: Integer;
  Mess: String;
  Index: Integer;
begin
  if (FPanelLeft.ScanCompleted) and (FPanelRight.ScanCompleted) then
  begin
    tmWait.Enabled := False;
    if not Application.Terminated then
    begin
      CommonCount := 0;
      pnStatus.Caption := 'Comparing files. Please wait...';
      for I := 0 to FPanelLeft.HashList.Count - 1 do
      begin
        if (bScan.Caption = 'Scan') or (Application.Terminated) then
          Break;
        Application.ProcessMessages;
        Index := FPanelRight.HashList.FindIndexOf(FPanelLeft.HashList.NameOfIndex(I));
        if Index > -1 then
        begin
          if (TFileData(FPanelLeft.HashList.Items[I]^).FSize = TFileData(FPanelRight.HashList.Items[Index]^).FSize) then
          begin
            Inc(CommonCount);
            FPanelLeft.lbFiles.TopIndex := FPanelLeft.lbFiles.Items.Add(TFileData(FPanelLeft.HashList.Items[I]^).FName); // + ' (' +  FPanelLeft.ThousandSeparator(IntToStr(TFileData(FPanelLeft.HashList.Items[I]^).FSize)) +  ' bytes)');
            FPanelRight.lbFiles.TopIndex := FPanelRight.lbFiles.Items.Add(TFileData(FPanelRight.HashList.Items[Index]^).FName); // + ' (' +  FPanelRight.ThousandSeparator(IntToStr(TFileData(FPanelRight.HashList.Items[Index]^).FSize)) +  ' bytes)');
          end;
        end;
      end;
      Mess := 'Scan completed in ' + IntToStr(SecondsBetween(Now, FScanTime)) +  ' seconds.' + sLineBreak +
              'Files found(Left: ' + IntToStr(FPanelLeft.HashList.Count) + ', Right: ' + IntToStr(FPanelRight.HashList.Count) + ')' + sLineBreak +
              'Common files: ' + IntToStr(CommonCount);
      bScan.Caption := 'Scan';
      pnStatus.Caption := 'Common files: ' + IntToStr(CommonCount);
      MessageDlg(Mess, mtInformation, [mbOk], 0);
    end;
  end;
end;

procedure TfMain.ReadSettings;
var
  IniFile: TIniFile;
begin
  if (not DirectoryExists(FConfigDir)) or (not FileExists(FConfigFile)) then
    Exit;

  IniFile := TIniFile.Create(FConfigFile);
  try
    FPanelLeft.edPath.Text  := IniFile.ReadString('APath', 'Left', '');
    FPanelRight.edPath.Text := IniFile.ReadString('APath', 'Right', '');
  finally
    IniFile.Free;
  end;
end;

procedure TfMain.WriteSettings;
var
  IniFile: TIniFile;
begin
  if (not DirectoryExists(FConfigDir)) then
    if (not CreateDir(FConfigDir)) then
      Exit;

  IniFile := TIniFile.Create(FConfigFile);
  try
    IniFile.WriteString('APath', 'Left', FPanelLeft.edPath.Text);
    IniFile.WriteString('APath', 'Right', FPanelRight.edPath.Text);
  finally
    IniFile.Free;
  end;
end;

procedure TfMain.ListBoxOnScroll(Sender: TObject);
begin
  if ((Sender as TScrolListBox).Parent.Parent is TfrPanel) then
   begin
     case ((Sender as TScrolListBox).Parent.Parent as TfrPanel).PanelType of
       frPanelLeft: FPanelRight.LBFiles.TopIndex := FPanelLeft.LBFiles.TopIndex;
       frPanelRight: FPanelLeft.LBFiles.TopIndex := FPanelRight.LBFiles.TopIndex;
     end;
   end;
end;

procedure TfMain.ListBoxOnSelectionChange(Sender: TObject; User: boolean);
begin
  if ((Sender as TScrolListBox).Parent.Parent is TfrPanel) then
  begin
    case ((Sender as TScrolListBox).Parent.Parent as TfrPanel).PanelType of
      frPanelLeft: FPanelRight.LBFiles.ItemIndex := FPanelLeft.LBFiles.ItemIndex;
      frPanelRight: FPanelLeft.LBFiles.ItemIndex := FPanelRight.LBFiles.ItemIndex;
    end;
  end;
end;

end.

