unit uPanel;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, ExtCtrls, StdCtrls, Dialogs,
  EditBtn, Messages, contnrs, uFileSearch;

type
  TfrPanelType = (frPanelLeft, frPanelRight);

  { TfrPanel }

  { TScrolListBox }

  TScrolListBox = class(TListBox)
  private
    FOnScroll: TNotifyEvent;
    procedure Scroll(var Msg: TMessage); message WM_VSCROLL;
  published
    property OnScroll: TNotifyEvent read FOnScroll write FOnScroll;
  end;

  TfrPanel = class(TFrame)
    edPath: TEditButton;
    lbFileCount: TLabel;
    pnBottom: TPanel;
    pnFiles: TPanel;
    pnLeft: TPanel;
    pnRight: TPanel;
    pnTop: TPanel;
    SelectDirectory: TSelectDirectoryDialog;
    procedure edPathButtonClick(Sender: TObject);
  private
    FScanCompleted: boolean;
    FPanelType: TfrPanelType;
    FFileSearch: TFileSearch;
    FHashList: TFPHashList;
    FLBFiles: TScrolListBox;
    procedure OnSearching(Sender: TObject; AFileCount: integer);
    procedure OnSearchComplete(Sender: TObject; AHashList: TFPHashList);
    procedure ClearHashList;
    { private declarations }
  public
    procedure InitFrame(const APanelType: TfrPanelType; const AParent: TWinControl);
    procedure FinalizeFrame;
    function CanScan: boolean;
    procedure Scan(const AStop: boolean = False);
    function ThousandSeparator(const ANumber: string): string;
  public
    property ScanCompleted: boolean read FScanCompleted write FScanCompleted;
    property HashList: TFPHashList read FHashList write FHashList;
    property LBFiles: TScrolListBox read FLBFiles;
    property PanelType: TfrPanelType read FPanelType;
    { public declarations }
  end;

implementation

{$R *.lfm}

{ TScrolListBox }

procedure TScrolListBox.Scroll(var Msg: TMessage);
begin
  inherited;
  if Assigned(FOnScroll) then
    FOnScroll(Self);
end;

{ TfrPanel }

procedure TfrPanel.edPathButtonClick(Sender: TObject);
begin
  if SelectDirectory.Execute then
    edPath.Text := SelectDirectory.FileName;
end;

procedure TfrPanel.OnSearching(Sender: TObject; AFileCount: integer);
begin
  lbFileCount.Caption := 'Files found: ' + IntToStr(AFileCount);
  lbFileCount.Invalidate;
end;

procedure TfrPanel.OnSearchComplete(Sender: TObject; AHashList: TFPHashList);
begin
  FScanCompleted := True;
  if not Application.Terminated then
  begin
    FHashList := AHashList;
    lbFileCount.Caption := 'Files found: ' + IntToStr(AHashList.Count) + ' (Completed)';
    lbFileCount.Invalidate;
  end;
end;

procedure TfrPanel.ClearHashList;
var
  FileData: PFileData;
  I: integer;
begin
  for I := 0 to FHashList.Count - 1 do
  begin
    FileData := FHashList.Items[I];
    Dispose(FileData);
  end;
  FHashList.Clear;
end;

function TfrPanel.ThousandSeparator(const ANumber: string): string;
var
  Str, StrAux: string;
  I, K: integer;
begin
  Str := ANumber;
  StrAux := '';
  for I := Length(Str) downto 1 do
    StrAux := StrAux + Str[i];
  K := 0;
  for I := 1 to length(StrAux) do
  begin
    if (I mod 3 = 0) and (I <> length(StrAux) - K) then
    begin
      Inc(K);
      Insert(',', StrAux, I + K);
    end;
    Str := '';
  end;
  for I := length(StrAux) downto 1 do
    Str := Str + StrAux[I];
  Result := Str;
end;

function TfrPanel.CanScan: boolean;
begin
  Result := True;
  if (Trim(edPath.Text) = '') or (not DirectoryExists(edPath.Text)) then
  begin
    if FPanelType = frPanelLeft then
      MessageDlg('Please enter a valid path(left panel)!', mtInformation, [mbOK], 0)
    else
      MessageDlg('Please enter a valid path(right panel)!', mtInformation, [mbOK], 0);
    edPath.SetFocus;
    Result := False;
    Exit;
  end;
end;

procedure TfrPanel.Scan(const AStop: boolean);
begin
  if not AStop then
  begin
    FScanCompleted := False;
    ClearHashList;
    lbFileCount.Caption := '';
    lbFiles.Items.BeginUpdate;
    lbFiles.Items.Clear;
    lbFiles.Items.EndUpdate;
    FFileSearch := TFileSearch.Create(edPath.Text, FHashList);
    FFileSearch.FreeOnTerminate := True;
    FFileSearch.OnSearching := @OnSearching;
    FFileSearch.OnSearchComplete := @OnSearchComplete;
    FFileSearch.Start;
  end
  else
  begin
    FFileSearch.NeedToBreak := True;
    FScanCompleted := True;
  end;
end;

procedure TfrPanel.InitFrame(const APanelType: TfrPanelType; const AParent: TWinControl);
begin
  FLBFiles := TScrolListBox.Create(pnFiles);
  FLBFiles.Parent := pnFiles;
  FLBFiles.Align := alClient;
  FHashList := TFPHashList.Create;
  FPanelType := APanelType;
  Parent := AParent;
  Width := 500;
  case FPanelType of
    frPanelLeft:
    begin
      Name := 'pnLeft';
      Align := alLeft;
      pnRight.Visible := False;
    end;
    frPanelRight:
    begin
      Name := 'pnRight';
      Align := alClient;
      pnLeft.Visible := False;
      edPath.Left := 0;
      lbFileCount.Left := 0;
    end;
  end;
end;

procedure TfrPanel.FinalizeFrame;
begin
  if Assigned(FFileSearch) then
    FFileSearch.NeedToBreak := True;
  ClearHashList;
  FHashList.Free;
end;


end.
